import { Injectable, Inject, PLATFORM_ID } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';

import {
  LoggerConfig,
  LoggerLevel,
  LoggerLevelName,
  _LOGGER_CONFIG,
} from '../core';

@Injectable()
export class LoggerService {
  private _isIE: boolean;

  public constructor(
    @Inject(_LOGGER_CONFIG) private readonly config: LoggerConfig,
    @Inject(PLATFORM_ID) private readonly platformId,
  ) {
    this._isIE = isPlatformBrowser(platformId) &&
    !!(navigator.userAgent.indexOf('MSIE') !== -1 || navigator.userAgent.match(/Trident\//) || navigator.userAgent.match(/Edge\//));
  }

  public trace(message, ...additional: any[]): void {
    this._log(LoggerLevel.TRACE, message, additional);
  }

  public debug(message, ...additional: any[]): void {
    this._log(LoggerLevel.DEBUG, message, additional);
  }

  public info(message, ...additional: any[]): void {
    this._log(LoggerLevel.INFO, message, additional);
  }

  public log(message, ...additional: any[]): void {
    this._log(LoggerLevel.LOG, message, additional);
  }

  public warn(message, ...additional: any[]): void {
    this._log(LoggerLevel.WARN, message, additional);
  }

  public error(message, ...additional: any[]): void {
    this._log(LoggerLevel.ERROR, message, additional);
  }

  private _timestamp(): string {
    return new Date().toISOString();
  }

  private _log(level: LoggerLevel, message, additional: any[] = []): void {
    if (!message) {
      return;
    }

    // Allow logging on server even if client log level is off
    // if (logOnServer) {
    //   this._logOnServer(level, message, additional);
    // }

    // if no message or the log level is less than the environ
    if (level < this.config.level) {
      return;
    }

    try {
      message = typeof message === 'string' ? message : JSON.stringify(message, null, 2);
    } catch (e) {
      additional = [message, ...additional];
      message = 'The provided "message" value could not be parsed with JSON.stringify().';
    }

    // Coloring doesn't work in IE
    if (this._isIE) {
      return this._logIE(level, message, additional);
    }

    const color = this._getColor(level);

    console.log(`%c${this._timestamp()} [${LoggerLevelName[level]}]`, `color:${color}`, message, ...additional);
  }

  private _logIE(level: LoggerLevel, message: string, additional: any[] = []): void {
    switch (level) {
      case LoggerLevel.WARN:
        console.warn(`${this._timestamp()} [${LoggerLevelName[level]}] `, message, ...additional);
        break;
      case LoggerLevel.ERROR:
        console.error(`${this._timestamp()} [${LoggerLevelName[level]}] `, message, ...additional);
        break;
      case LoggerLevel.INFO:
        console.log(`${this._timestamp()} [${LoggerLevelName[level]}] `, message, ...additional);
        break;
      default:
        console.log(`${this._timestamp()} [${LoggerLevelName[level]}] `, message, ...additional);
    }
  }

  private _getColor(level: LoggerLevel): 'blue' | 'teal' | 'gray' | 'red' | undefined {
    switch (level) {
      case LoggerLevel.TRACE:
        return 'blue';
      case LoggerLevel.DEBUG:
        return 'teal';
      case LoggerLevel.INFO:
      case LoggerLevel.LOG:
        return 'gray';
      case LoggerLevel.WARN:
      case LoggerLevel.ERROR:
        return 'red';
      case LoggerLevel.OFF:
      default:
        return;
    }
  }
}
